# -*- coding: utf-8 -*-
#--------------------------------
#$Date: 2008-08-07 01:53:57 +0600 (Чтв, 07 Авг 2008) $
#$Author: daev $
#$Revision: 1792 $
#--------------------------------
#Copyright (C) 2007-2008 Alexander Koshelev (daevaorn@gmail.com)
from django.utils.functional import curry
from django.core.cache import cache
from django.utils.encoding import smart_str
from django.utils.itercompat import is_iterable

import md5

def to_list(arg):
    """Converts `arg` to list"""
    if is_iterable(arg) and not isinstance(arg, dict):
        return list(arg)
    else:
        return [arg]

def make_cache_key(base, suffix_list):
    return md5.new(":".join(map(smart_str, [base] + suffix_list))).hexdigest()

def make_cache_key_keylist(base):
    return "%s_keys" % base

def delete_by_stored_keys(cache_key_keylist):
    keylist = cache.get(cache_key_keylist)
    if isinstance(keylist, list):
        for k in keylist:
            cache.delete(k)

def invalidate(store_keys, cache_key, cache_key_keylist):
    if store_keys:
        delete_by_stored_keys(cache_key_keylist)
    else:
        cache.delete(cache_key)

def connect_invalidator(triggers, cache_key, cache_key_suffix, store_keys, cache_key_keylist):
    """Connects invalidator to all needed signals"""
    defaults = {"suffix" : lambda *args, **kwargs: [],
                "checker": None,
                "signal": []
            }

    triggers_list = to_list(triggers)
    for t in triggers_list:
        trigger = defaults.copy()
        trigger.update(t)

        suffix_getter = trigger["suffix"]
        checker       = trigger["checker"]
        sender        = trigger["sender"]

        signals       = trigger["signal"]

        for signal in to_list(signals):
            def make_cache_invalidator(suffix_getter):
                def cache_invalidator(signal, sender, *args, **kwargs):
                    if checker is not None:
                        if checker(*args, **kwargs):
                            invalidate(store_keys, cache_key, cache_key_keylist)
                        else:
                            return
                    else:
                        if cache_key_suffix == to_list(suffix_getter(*args, **kwargs)):
                            invalidate(store_keys, cache_key, cache_key_keylist)
                return cache_invalidator

            signal.connect(make_cache_invalidator(suffix_getter),
                            sender=sender,
                            weak=False,
                            dispatch_uid="%s%s" % (cache_key, triggers_list.index(t))
                        )

class CacheWrapper(object):
    def __init__(self, func, trigger, suffix, base_name, store_keys):
        self.trigger = trigger
        self.suffix = suffix and suffix or (lambda *args, **kwargs:[])

        self.func = func
        self.base_name = base_name
        self.store_keys = store_keys

    def __call__(self, *args, **kwargs):
        cache_key_suffix = to_list(self.suffix(*args, **kwargs))
        cache_key = make_cache_key(self.base_name, cache_key_suffix)
        cache_key_keylist = make_cache_key_keylist(self.base_name)

        value = cache.get(cache_key)

        connect_invalidator(self.trigger, cache_key, cache_key_suffix, self.store_keys, cache_key_keylist)

        if value is None:
            value = self.func(*args, **kwargs)
            cache.set(cache_key, value)

            if self.store_keys:
                keylist = cache.get(cache_key_keylist)
                if isinstance(keylist, list):
                    keylist.append(cache_key)
                    cache.set(cache_key_keylist, keylist)
                else:
                    cache.set(cache_key_keylist, [cache_key])

        return value
